// Generated automatically with "cito". Do not edit.
package net.sf.asap;

/**
 * Atari 8-bit chip music emulator.
 * This class performs no I/O operations - all music data must be passed in byte arrays.
 */
public class ASAP
{
	public ASAP()
	{
		this.silenceCycles = 0;
		this.cpu.asap = this;
	}
	private int blocksPlayed;

	private void call6502(int addr)
	{
		this.cpu.memory[53760] = 32;
		this.cpu.memory[53761] = (byte) addr;
		this.cpu.memory[53762] = (byte) (addr >> 8);
		this.cpu.memory[53763] = -46;
		this.cpu.pc = 53760;
	}

	private void call6502Player()
	{
		int player = this.moduleInfo.player;
		switch (this.moduleInfo.type) {
		case ASAPModuleType.SAP_B:
			this.call6502(player);
			break;
		case ASAPModuleType.SAP_C:
		case ASAPModuleType.CMC:
		case ASAPModuleType.CM3:
		case ASAPModuleType.CMR:
		case ASAPModuleType.CMS:
			this.call6502(player + 6);
			break;
		case ASAPModuleType.SAP_D:
			if (player >= 0) {
				this.cpu.pushPc();
				this.cpu.memory[53760] = 8;
				this.cpu.memory[53761] = 72;
				this.cpu.memory[53762] = -118;
				this.cpu.memory[53763] = 72;
				this.cpu.memory[53764] = -104;
				this.cpu.memory[53765] = 72;
				this.cpu.memory[53766] = 32;
				this.cpu.memory[53767] = (byte) player;
				this.cpu.memory[53768] = (byte) (player >> 8);
				this.cpu.memory[53769] = 104;
				this.cpu.memory[53770] = -88;
				this.cpu.memory[53771] = 104;
				this.cpu.memory[53772] = -86;
				this.cpu.memory[53773] = 104;
				this.cpu.memory[53774] = 64;
				this.cpu.pc = 53760;
			}
			break;
		case ASAPModuleType.SAP_S:
			int i = (this.cpu.memory[69] & 0xff) - 1;
			this.cpu.memory[69] = (byte) i;
			if (i == 0)
				this.cpu.memory[45179] = (byte) ((this.cpu.memory[45179] & 0xff) + 1);
			break;
		case ASAPModuleType.DLT:
			this.call6502(player + 259);
			break;
		case ASAPModuleType.MPT:
		case ASAPModuleType.RMT:
		case ASAPModuleType.TM2:
		case ASAPModuleType.FC:
			this.call6502(player + 3);
			break;
		case ASAPModuleType.TMC:
			if (--this.tmcPerFrameCounter <= 0) {
				this.tmcPerFrameCounter = this.cpu.memory[this.moduleInfo.getMusicAddress() + 31] & 0xff;
				this.call6502(player + 3);
			}
			else
				this.call6502(player + 6);
			break;
		}
	}
	private int consol;
	private final byte[] covox = new byte[4];
	private final Cpu6502 cpu = new Cpu6502();
	private int currentDuration;
	private int currentSong;

	/**
	 * Enables silence detection.
	 * Causes playback to stop after the specified period of silence.
	 * @param seconds Length of silence which ends playback. Zero disables silence detection.
	 */
	public final void detectSilence(int seconds)
	{
		this.silenceCyclesCounter = this.silenceCycles = seconds * 1773447;
	}

	private int do6502Frame()
	{
		this.nextEventCycle = 0;
		this.nextScanlineCycle = 0;
		this.nmist = this.nmist == NmiStatus.RESET ? NmiStatus.ON_V_BLANK : NmiStatus.WAS_V_BLANK;
		int cycles = this.moduleInfo.isNtsc() ? 29868 : 35568;
		this.cpu.doFrame(cycles);
		this.cpu.cycle -= cycles;
		if (this.nextPlayerCycle != 8388608)
			this.nextPlayerCycle -= cycles;
		for (int i = 3;; i >>= 1) {
			this.pokeys.basePokey.channels[i].endFrame(cycles);
			this.pokeys.extraPokey.channels[i].endFrame(cycles);
			if (i == 0)
				break;
		}
		return cycles;
	}

	private void do6502Init(int pc, int a, int x, int y) throws Exception
	{
		this.cpu.pc = pc;
		this.cpu.a = a & 255;
		this.cpu.x = x & 255;
		this.cpu.y = y & 255;
		this.cpu.memory[53760] = -46;
		this.cpu.memory[510] = -1;
		this.cpu.memory[511] = -47;
		this.cpu.s = 253;
		for (int frame = 0; frame < 50; frame++) {
			this.do6502Frame();
			if (this.cpu.pc == 53760)
				return;
		}
		throw new Exception("INIT routine didn't return");
	}

	private int doFrame()
	{
		this.gtiaOrCovoxPlayedThisFrame = false;
		this.pokeys.startFrame();
		int cycles = this.do6502Frame();
		this.pokeys.endFrame(cycles);
		return cycles;
	}

	private static int fourCC(String s)
	{
		return s.charAt(0) | s.charAt(1) << 8 | s.charAt(2) << 16 | s.charAt(3) << 24;
	}

	/**
	 * Fills the specified buffer with generated samples.
	 * @param buffer The destination buffer.
	 * @param bufferLen Number of bytes to fill.
	 * @param format Format of samples.
	 */
	public final int generate(byte[] buffer, int bufferLen, int format)
	{
		return this.generateAt(buffer, 0, bufferLen, format);
	}

	private int generateAt(byte[] buffer, int bufferOffset, int bufferLen, int format)
	{
		if (this.silenceCycles > 0 && this.silenceCyclesCounter <= 0)
			return 0;
		int blockShift = this.moduleInfo.getChannels() - 1 + (format != ASAPSampleFormat.U8 ? 1 : 0);
		int bufferBlocks = bufferLen >> blockShift;
		if (this.currentDuration > 0) {
			int totalBlocks = ASAP.millisecondsToBlocks(this.currentDuration);
			if (bufferBlocks > totalBlocks - this.blocksPlayed)
				bufferBlocks = totalBlocks - this.blocksPlayed;
		}
		int block = 0;
		for (;;) {
			int blocks = this.pokeys.generate(buffer, bufferOffset + (block << blockShift), bufferBlocks - block, format);
			this.blocksPlayed += blocks;
			block += blocks;
			if (block >= bufferBlocks)
				break;
			int cycles = this.doFrame();
			if (this.silenceCycles > 0) {
				if (this.pokeys.isSilent() && !this.gtiaOrCovoxPlayedThisFrame) {
					this.silenceCyclesCounter -= cycles;
					if (this.silenceCyclesCounter <= 0)
						break;
				}
				else
					this.silenceCyclesCounter = this.silenceCycles;
			}
		}
		return block << blockShift;
	}

	/**
	 * Returns current playback position in blocks.
	 * A block is one sample or a pair of samples for stereo.
	 */
	public final int getBlocksPlayed()
	{
		return this.blocksPlayed;
	}

	/**
	 * Returns information about the loaded module.
	 */
	public final ASAPInfo getInfo()
	{
		return this.moduleInfo;
	}

	/**
	 * Returns POKEY channel volume - an integer between 0 and 15.
	 * @param channel POKEY channel number (from 0 to 7).
	 */
	public final int getPokeyChannelVolume(int channel)
	{
		Pokey pokey = (channel & 4) == 0 ? this.pokeys.basePokey : this.pokeys.extraPokey;
		return pokey.channels[channel & 3].audc & 15;
	}

	/**
	 * Returns current playback position in milliseconds.
	 */
	public final int getPosition()
	{
		return this.blocksPlayed * 10 / 441;
	}

	/**
	 * Fills leading bytes of the specified buffer with WAV file header.
	 * Returns the number of changed bytes.
	 * @param buffer The destination buffer.
	 * @param format Format of samples.
	 * @param metadata Include metadata (title, author, date).
	 */
	public final int getWavHeader(byte[] buffer, int format, boolean metadata)
	{
		int use16bit = format != ASAPSampleFormat.U8 ? 1 : 0;
		int blockSize = this.moduleInfo.getChannels() << use16bit;
		int bytesPerSecond = 44100 * blockSize;
		int totalBlocks = ASAP.millisecondsToBlocks(this.currentDuration);
		int nBytes = (totalBlocks - this.blocksPlayed) * blockSize;
		ASAP.putLittleEndian(buffer, 8, 1163280727);
		ASAP.putLittleEndians(buffer, 12, 544501094, 16);
		buffer[20] = 1;
		buffer[21] = 0;
		buffer[22] = (byte) this.moduleInfo.getChannels();
		buffer[23] = 0;
		ASAP.putLittleEndians(buffer, 24, 44100, bytesPerSecond);
		buffer[32] = (byte) blockSize;
		buffer[33] = 0;
		buffer[34] = (byte) (8 << use16bit);
		buffer[35] = 0;
		int i = 36;
		if (metadata) {
			int year = this.moduleInfo.getYear();
			if (this.moduleInfo.getTitle().length() > 0 || this.moduleInfo.getAuthor().length() > 0 || year > 0) {
				ASAP.putLittleEndian(buffer, 44, 1330007625);
				i = ASAP.putWavMetadata(buffer, 48, 1296125513, this.moduleInfo.getTitle());
				i = ASAP.putWavMetadata(buffer, i, 1414676809, this.moduleInfo.getAuthor());
				if (year > 0) {
					ASAP.putLittleEndians(buffer, i, 1146241865, 6);
					for (int j = 3; j >= 0; j--) {
						buffer[i + 8 + j] = (byte) (48 + year % 10);
						year /= 10;
					}
					buffer[i + 12] = 0;
					buffer[i + 13] = 0;
					i += 14;
				}
				ASAP.putLittleEndians(buffer, 36, 1414744396, i - 44);
			}
		}
		ASAP.putLittleEndians(buffer, 0, 1179011410, i + nBytes);
		ASAP.putLittleEndians(buffer, i, 1635017060, nBytes);
		return i + 8;
	}
	private boolean gtiaOrCovoxPlayedThisFrame;

	final void handleEvent()
	{
		int cycle = this.cpu.cycle;
		if (cycle >= this.nextScanlineCycle) {
			if (cycle - this.nextScanlineCycle < 50)
				this.cpu.cycle = cycle += 9;
			this.nextScanlineCycle += 114;
			if (cycle >= this.nextPlayerCycle) {
				this.call6502Player();
				this.nextPlayerCycle += 114 * this.moduleInfo.getPlayerRateScanlines();
			}
		}
		int nextEventCycle = this.nextScanlineCycle;
		nextEventCycle = this.pokeys.basePokey.checkIrq(cycle, nextEventCycle);
		nextEventCycle = this.pokeys.extraPokey.checkIrq(cycle, nextEventCycle);
		this.nextEventCycle = nextEventCycle;
	}

	final boolean isIrq()
	{
		return this.pokeys.basePokey.irqst != 255;
	}

	/**
	 * Loads music data ("module").
	 * @param filename Filename, used to determine the format.
	 * @param module Contents of the file.
	 * @param moduleLen Length of the file.
	 */
	public final void load(String filename, byte[] module, int moduleLen) throws Exception
	{
		this.moduleInfo.load(filename, module, moduleLen);
		byte[] playerRoutine = ASAP6502.getPlayerRoutine(this.moduleInfo);
		if (playerRoutine != null) {
			int player = ASAPInfo.getWord(playerRoutine, 2);
			int playerLastByte = ASAPInfo.getWord(playerRoutine, 4);
			int music = this.moduleInfo.getMusicAddress();
			if (music <= playerLastByte)
				throw new Exception("Module address conflicts with the player routine");
			this.cpu.memory[19456] = 0;
			if (this.moduleInfo.type == ASAPModuleType.FC)
				System.arraycopy(module, 0, this.cpu.memory, music, moduleLen);
			else
				System.arraycopy(module, 6, this.cpu.memory, music, moduleLen - 6);
			System.arraycopy(playerRoutine, 6, this.cpu.memory, player, playerLastByte + 1 - player);
			if (this.moduleInfo.player < 0)
				this.moduleInfo.player = player;
			return;
		}
		clear(this.cpu.memory);
		int moduleIndex = this.moduleInfo.headerLen + 2;
		while (moduleIndex + 5 <= moduleLen) {
			int startAddr = ASAPInfo.getWord(module, moduleIndex);
			int blockLen = ASAPInfo.getWord(module, moduleIndex + 2) + 1 - startAddr;
			if (blockLen <= 0 || moduleIndex + blockLen > moduleLen)
				throw new Exception("Invalid binary block");
			moduleIndex += 4;
			System.arraycopy(module, moduleIndex, this.cpu.memory, startAddr, blockLen);
			moduleIndex += blockLen;
			if (moduleIndex == moduleLen)
				return;
			if (moduleIndex + 7 <= moduleLen && module[moduleIndex] == -1 && module[moduleIndex + 1] == -1)
				moduleIndex += 2;
		}
		throw new Exception("Invalid binary block");
	}

	private static int millisecondsToBlocks(int milliseconds)
	{
		return milliseconds * 441 / 10;
	}
	private final ASAPInfo moduleInfo = new ASAPInfo();

	/**
	 * Mutes the selected POKEY channels.
	 * @param mask An 8-bit mask which selects POKEY channels to be muted.
	 */
	public final void mutePokeyChannels(int mask)
	{
		this.pokeys.basePokey.mute(mask);
		this.pokeys.extraPokey.mute(mask >> 4);
	}
	int nextEventCycle;
	private int nextPlayerCycle;
	private int nextScanlineCycle;
	private int nmist;

	final int peekHardware(int addr)
	{
		switch (addr & 65311) {
		case 53268:
			return this.moduleInfo.isNtsc() ? 15 : 1;
		case 53279:
			return ~this.consol & 15;
		case 53770:
		case 53786:
		case 53774:
		case 53790:
			return this.pokeys.peek(addr, this.cpu.cycle);
		case 53772:
		case 53788:
		case 53775:
		case 53791:
			return 255;
		case 54283:
		case 54299:
			int cycle = this.cpu.cycle;
			if (cycle > (this.moduleInfo.isNtsc() ? 29868 : 35568))
				return 0;
			return cycle / 228;
		case 54287:
		case 54303:
			switch (this.nmist) {
			case NmiStatus.RESET:
				return 31;
			case NmiStatus.WAS_V_BLANK:
				return 95;
			default:
				return this.cpu.cycle < 28291 ? 31 : 95;
			}
		default:
			return this.cpu.memory[addr] & 0xff;
		}
	}

	/**
	 * Prepares playback of the specified song of the loaded module.
	 * @param song Zero-based song index.
	 * @param duration Playback time in milliseconds, -1 means infinity.
	 */
	public final void playSong(int song, int duration) throws Exception
	{
		if (song < 0 || song >= this.moduleInfo.getSongs())
			throw new Exception("Song number out of range");
		this.currentSong = song;
		this.currentDuration = duration;
		this.nextPlayerCycle = 8388608;
		this.blocksPlayed = 0;
		this.silenceCyclesCounter = this.silenceCycles;
		this.cpu.reset();
		this.nmist = NmiStatus.ON_V_BLANK;
		this.consol = 8;
		this.covox[0] = -128;
		this.covox[1] = -128;
		this.covox[2] = -128;
		this.covox[3] = -128;
		this.pokeys.initialize(this.moduleInfo.isNtsc(), this.moduleInfo.getChannels() > 1);
		this.mutePokeyChannels(255);
		int player = this.moduleInfo.player;
		int music = this.moduleInfo.getMusicAddress();
		switch (this.moduleInfo.type) {
		case ASAPModuleType.SAP_B:
			this.do6502Init(this.moduleInfo.getInitAddress(), song, 0, 0);
			break;
		case ASAPModuleType.SAP_C:
		case ASAPModuleType.CMC:
		case ASAPModuleType.CM3:
		case ASAPModuleType.CMR:
		case ASAPModuleType.CMS:
			this.do6502Init(player + 3, 112, music, music >> 8);
			this.do6502Init(player + 3, 0, song, 0);
			break;
		case ASAPModuleType.SAP_D:
		case ASAPModuleType.SAP_S:
			this.cpu.pc = this.moduleInfo.getInitAddress();
			this.cpu.a = song;
			this.cpu.x = 0;
			this.cpu.y = 0;
			this.cpu.s = 255;
			break;
		case ASAPModuleType.DLT:
			this.do6502Init(player + 256, 0, 0, this.moduleInfo.songPos[song] & 0xff);
			break;
		case ASAPModuleType.MPT:
			this.do6502Init(player, 0, music >> 8, music);
			this.do6502Init(player, 2, this.moduleInfo.songPos[song] & 0xff, 0);
			break;
		case ASAPModuleType.RMT:
			this.do6502Init(player, this.moduleInfo.songPos[song] & 0xff, music, music >> 8);
			break;
		case ASAPModuleType.TMC:
		case ASAPModuleType.TM2:
			this.do6502Init(player, 112, music >> 8, music);
			this.do6502Init(player, 0, song, 0);
			this.tmcPerFrameCounter = 1;
			break;
		case ASAPModuleType.FC:
			this.do6502Init(player, song, 0, 0);
			break;
		}
		this.mutePokeyChannels(0);
		this.nextPlayerCycle = 0;
	}

	final void pokeHardware(int addr, int data)
	{
		if (addr >> 8 == 210) {
			int t = this.pokeys.poke(addr, data, this.cpu.cycle);
			if (this.nextEventCycle > t)
				this.nextEventCycle = t;
		}
		else if ((addr & 65295) == 54282) {
			int x = this.cpu.cycle % 114;
			this.cpu.cycle += (x <= 106 ? 106 : 220) - x;
		}
		else if ((addr & 65295) == 54287) {
			this.nmist = this.cpu.cycle < 28292 ? NmiStatus.ON_V_BLANK : NmiStatus.RESET;
		}
		else if ((addr & 65280) == this.moduleInfo.getCovoxAddress()) {
			Pokey pokey;
			addr &= 3;
			if (addr == 0 || addr == 3)
				pokey = this.pokeys.basePokey;
			else
				pokey = this.pokeys.extraPokey;
			int delta = data - (this.covox[addr] & 0xff);
			if (delta != 0) {
				pokey.addDelta(this.pokeys, this.cpu.cycle, delta << 17);
				this.covox[addr] = (byte) data;
				this.gtiaOrCovoxPlayedThisFrame = true;
			}
		}
		else if ((addr & 65311) == 53279) {
			int delta = (this.consol & 8) - (data & 8) << 20;
			if (delta != 0) {
				int cycle = this.cpu.cycle;
				this.pokeys.basePokey.addDelta(this.pokeys, cycle, delta);
				this.pokeys.extraPokey.addDelta(this.pokeys, cycle, delta);
				this.gtiaOrCovoxPlayedThisFrame = true;
			}
			this.consol = data;
		}
		else
			this.cpu.memory[addr] = (byte) data;
	}
	private final PokeyPair pokeys = new PokeyPair();

	private static void putLittleEndian(byte[] buffer, int offset, int value)
	{
		buffer[offset] = (byte) value;
		buffer[offset + 1] = (byte) (value >> 8);
		buffer[offset + 2] = (byte) (value >> 16);
		buffer[offset + 3] = (byte) (value >> 24);
	}

	private static void putLittleEndians(byte[] buffer, int offset, int value1, int value2)
	{
		ASAP.putLittleEndian(buffer, offset, value1);
		ASAP.putLittleEndian(buffer, offset + 4, value2);
	}

	private static int putWavMetadata(byte[] buffer, int offset, int fourCC, String value)
	{
		int len = value.length();
		if (len > 0) {
			ASAP.putLittleEndians(buffer, offset, fourCC, (len | 1) + 1);
			offset += 8;
			for (int i = 0; i < len; i++)
				buffer[offset++] = (byte) value.charAt(i);
			buffer[offset++] = 0;
			if ((len & 1) == 0)
				buffer[offset++] = 0;
		}
		return offset;
	}
	/**
	 * Output sample rate.
	 */
	public static final int SAMPLE_RATE = 44100;

	/**
	 * Changes the playback position.
	 * @param position The requested absolute position in milliseconds.
	 */
	public final void seek(int position) throws Exception
	{
		this.seekSample(ASAP.millisecondsToBlocks(position));
	}

	/**
	 * Changes the playback position.
	 * @param block The requested absolute position in samples (always 44100 per second, even in stereo).
	 */
	public final void seekSample(int block) throws Exception
	{
		if (block < this.blocksPlayed)
			this.playSong(this.currentSong, this.currentDuration);
		while (this.blocksPlayed + this.pokeys.readySamplesEnd < block) {
			this.blocksPlayed += this.pokeys.readySamplesEnd;
			this.doFrame();
		}
		this.pokeys.readySamplesStart = block - this.blocksPlayed;
		this.blocksPlayed = block;
	}
	private int silenceCycles;
	private int silenceCyclesCounter;
	private int tmcPerFrameCounter;
	private static void clear(byte[] array)
	{
		for (int i = 0; i < array.length; i++)
			array[i] = 0;
	}
}
